import sys

from je_api_testka.utils.test_record.record_test_result_class import test_record
from je_api_testka.utils.exception.exceptions import HTMLException
from je_api_testka.utils.exception.exceptions_tag import html_generate_no_data_tag
from threading import Lock

lock = Lock()

_html_string_head = \
    """
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8"/>
        <title>Load Density Report</title>

        <style>

            body{
                font-size: 100%;
            }

            h1{
                font-size: 2em;
            }

            .main_table {
                margin: 0 auto;
                border-collapse: collapse;
                width: 75%;
                font-size: 1.5em;
            }

            .success_table_head {
                border: 3px solid #262626;
                background-color: aqua;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
            }

            .failure_table_head {
                border: 3px solid #262626;
                background-color: #f84c5f;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
            }

            .table_data_field_title {
                border: 3px solid #262626;
                padding: 0;
                margin: 0;
                background-color: #dedede;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
                width: 25%;
            }

            .table_data_field_text {
                border: 3px solid #262626;
                padding: 0;
                margin: 0;
                background-color: #dedede;
                font-family: "Times New Roman", sans-serif;
                text-align: left;
                width: 75%;
            }

            .text {
                text-align: center;
                font-family: "Times New Roman", sans-serif;
            }
        </style>
    </head>
    <body>
    <h1 class="text">
        Test Report
    </h1>
    """.strip()

_html_string_bottom = \
    """
    </body>
    </html>
    """.strip()

success_table = \
    r"""
    <table class="main_table">
        <thead>
        <tr>
            <th colspan="2" class="success_table_head">Test Report</th>
        </tr>
        </thead>
        <tbody>
        <tr>
            <td class="table_data_field_title">status_code</td>
            <td class="table_data_field_text">{status_code}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">text</td>
            <td class="table_data_field_text">{text}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">content</td>
            <td class="table_data_field_text">{content}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">headers</td>
            <td class="table_data_field_text">{headers}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">history</td>
            <td class="table_data_field_text">{history}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">encoding</td>
            <td class="table_data_field_text">{encoding}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">cookies</td>
            <td class="table_data_field_text">{cookies}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">elapsed</td>
            <td class="table_data_field_text">{elapsed}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">request_time_sec</td>
            <td class="table_data_field_text">{request_time_sec}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">request_method</td>
            <td class="table_data_field_text">{request_method}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">request_url</td>
            <td class="table_data_field_text">{request_url}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">request_body</td>
            <td class="table_data_field_text">{request_body}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">start_time</td>
            <td class="table_data_field_text">{start_time}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">end_time</td>
            <td class="table_data_field_text">{end_time}</td>
        </tr>
        </tbody>
    </table>
    <br>
    """.strip()

failure_table = \
    r"""
    <table class="main_table">
    <thead>
    <tr>
        <th colspan="2" class="failure_table_head">Test Report</th>
    </tr>
    </thead>
    <tbody>
    <tr>
        <td class="table_data_field_title">http_method</td>
        <td class="table_data_field_text">{http_method}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">test_url</td>
        <td class="table_data_field_text">{test_url}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">soap</td>
        <td class="table_data_field_text">{soap}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">record_request_info</td>
        <td class="table_data_field_text">{record_request_info}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">clean_record</td>
        <td class="table_data_field_text">{clean_record}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">result_check_dict</td>
        <td class="table_data_field_text">{result_check_dict}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">error</td>
        <td class="table_data_field_text">{error}</td>
    </tr>
    </tbody>
</table>
<br>
    """.strip()


def generate_html(html_name: str = "default_name"):
    """
    :param html_name: save html file name
    :return: html_string
    """
    if len(test_record.record_list) == 0 and len(test_record.error_record_list) == 0:
        raise HTMLException(html_generate_no_data_tag)
    else:
        success_list = list()
        for record_data in test_record.record_list:
            success_list.append(
                success_table.format(
                    status_code=record_data.get("status_code"),
                    text=record_data.get("text"),
                    content=str(record_data.get("content"), encoding="utf-8"),
                    headers=record_data.get("headers"),
                    history=record_data.get("history"),
                    encoding=record_data.get("encoding"),
                    cookies=record_data.get("cookies"),
                    elapsed=record_data.get("elapsed"),
                    request_time_sec=record_data.get("request_time_sec"),
                    request_method=record_data.get("request_method"),
                    request_url=record_data.get("request_url"),
                    request_body=record_data.get("request_body"),
                    start_time=record_data.get("start_time"),
                    end_time=record_data.get("end_time"),
                )
            )
        failure_list = list()
        if len(test_record.error_record_list) == 0:
            pass
        else:
            for record_data in test_record.error_record_list:
                failure_list.append(
                    failure_table.format(
                        http_method=record_data[0].get("http_method"),
                        test_url=record_data[0].get("test_url"),
                        soap=record_data[0].get("soap"),
                        record_request_info=record_data[0].get("record_request_info"),
                        clean_record=record_data[0].get("clean_record"),
                        result_check_dict=record_data[0].get("result_check_dict"),
                        error=record_data[1]
                    ),
                )
        try:
            lock.acquire()
            with open(html_name + ".html", "w+") as file_to_write:
                file_to_write.writelines(
                    _html_string_head
                )
                for success in success_list:
                    file_to_write.write(success)
                for failure in failure_list:
                    file_to_write.write(failure)
                file_to_write.writelines(
                    _html_string_bottom
                )
        except Exception as error:
            print(repr(error), file=sys.stderr)
        finally:
            lock.release()
    return success_list, failure_list



# general

class APITesterException(Exception):
    pass


# get method
class APITesterGetDataException(APITesterException):
    pass


class APITesterGetException(APITesterException):
    pass


class APITesterGetJsonException(APITesterException):
    pass


# put method
class APITesterPutException(APITesterException):
    pass


# delete method
class APITesterDeleteException(APITesterException):
    pass


# put method
class APITesterPostException(APITesterException):
    pass


# head method
class APITesterHeadException(APITesterException):
    pass


# options method
class APITesterOptionsException(APITesterException):
    pass


# patch method
class APITesterPatchException(APITesterException):
    pass


# session method


class APITesterSessionException(APITesterException):
    pass


# execute


class APITesterExecuteException(APITesterException):
    pass


# json
class APITesterJsonException(APITesterException):
    pass


# xml
class APITesterXMLException(APITesterException):
    pass


class APITesterXMLTypeException(APITesterException):
    pass


# API data check
class APIStatusCodeException(APITesterException):
    pass


class APITextException(APITesterException):
    pass


class APIContentException(APITesterException):
    pass


class APIHeadersException(APITesterException):
    pass


class APIHistoryException(APITesterException):
    pass


class APIEncodingException(APITesterException):
    pass


class APICookiesException(APITesterException):
    pass


class APIElapsedException(APITesterException):
    pass


class APIRequestsMethodException(APITesterException):
    pass


class APIRequestsUrlException(APITesterException):
    pass


class APIRequestsBodyException(APITesterException):
    pass


# check exception

class APIAssertException(APITesterException):
    pass


# html exception

class HTMLException(APITesterException):
    pass



from typing import Tuple

try:
    from je_matplotlib_wrapper import set_tkinter_embed_matplotlib_barh
except ImportError as error:
    raise ImportError(repr(error) + " you need install je_matplotlib_wrapper to use this")

from tkinter import Tk

from je_api_testka.utils.test_record.record_test_result_class import test_record


def get_list(get_data_horizontal: str, get_data_vertical: str) -> Tuple[list, list]:
    list_vertical = list()
    list_horizontal = list()
    for i in test_record.record_list:
        list_vertical.append(i.get(get_data_vertical))
        list_horizontal.append(i.get(get_data_horizontal))
    return list_vertical, list_horizontal


def make_tkinter_request_time_graph():
    show_data = get_list("request_time_sec", "request_url")
    set_tkinter_embed_matplotlib_barh(
        y_content_list=show_data[0],
        x_content_list=show_data[1],
        show_figure_window=Tk(),
    )



import datetime
import sys

import requests

from je_api_testka.requests_wrapper.requests_http_method_wrapper import api_tester_method
from requests.structures import CaseInsensitiveDict

from je_api_testka.utils.exception.exceptions import APITesterGetDataException
from je_api_testka.utils.exception.exceptions import APITesterExecuteException
from je_api_testka.utils.get_data_strcture.get_api_data import get_api_response_data

from je_api_testka.utils.exception.exceptions_tag import api_test_get_data_error_message

from je_api_testka.utils.exception.exceptions_tag import api_test_get_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_put_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_delete_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_post_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_head_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_options_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_patch_error_message
from je_api_testka.utils.exception.exceptions_tag import api_test_session_error_message

from je_api_testka.utils.test_record.record_test_result_class import test_record

from je_api_testka.utils.assert_result.result_check import check_result

exception_message_dict = {
    "get": api_test_get_error_message,
    "put": api_test_put_error_message,
    "delete": api_test_delete_error_message,
    "post": api_test_post_error_message,
    "head": api_test_head_error_message,
    "options": api_test_options_error_message,
    "patch": api_test_patch_error_message,
    "session_get": api_test_session_error_message,
    "session_put": api_test_session_error_message,
    "session_patch": api_test_session_error_message,
    "session_post": api_test_session_error_message,
    "session_head": api_test_session_error_message,
    "session_delete": api_test_session_error_message,
    "session_options": api_test_session_error_message,

}


def get_response(response: requests.Response,
                 start_time: [str, float, int],
                 end_time: [str, float, int]) -> dict:
    """
    use requests response to create data dict
    :param response: requests response
    :param start_time: test start time
    :param end_time: test end time
    :return: data dict include [status_code, text, content, headers, history, encoding, cookies,
    elapsed, request_time_sec, request_method, request_url, request_body, start_time, end_time]
    """
    try:
        return get_api_response_data(response, start_time, end_time)
    except APITesterGetDataException:
        raise APITesterGetDataException(api_test_get_data_error_message)


def test_api_method(http_method: str, test_url: str,
                    soap: bool = False, record_request_info: bool = True,
                    clean_record: bool = False, result_check_dict: dict = None, **kwargs) \
        -> (requests.Response, dict):
    """
    set requests http_method url headers and record response and record report
    :param http_method:
    :param test_url:
    :param soap:
    :param record_request_info:
    :param clean_record:
    :param result_check_dict:
    :param kwargs:
    :return:
    """
    try:
        try:
            start_time = datetime.datetime.now()
            if soap is False:
                response = api_tester_method(http_method, test_url=test_url, **kwargs)
            else:
                headers = CaseInsensitiveDict()
                headers["Content-Type"] = "application/soap+xml"
                return test_api_method(http_method, test_url=test_url, headers=headers, **kwargs)
            end_time = datetime.datetime.now()
            response_data = get_response(response, start_time, end_time)
            if clean_record:
                test_record.clean_record()
            if result_check_dict is None:
                if record_request_info:
                    test_record.record_list.append(response_data)
                return {"response": response, "response_data": response_data}
            else:
                check_result(response_data, result_check_dict)
                if record_request_info:
                    test_record.record_list.append(response_data)
                return {"response": response, "response_data": response_data}
        except APITesterExecuteException as error:
            raise repr(error)
    except Exception as error:
        print(repr(error), file=sys.stderr)
        test_record.error_record_list.append([
            {
                "http_method": http_method,
                "test_url": test_url,
                "soap": soap,
                "record_request_info": record_request_info,
                "clean_record": clean_record,
                "result_check_dict": result_check_dict
            },
            repr(error)]
        )



from je_api_testka.utils.exception.exceptions import APIAssertException


def check_result(result_dict: dict, result_check_dict: dict):
    """
    :param result_dict: response result dict (get_api_response_data's return data)
    :param result_check_dict: the dict include data name and value to check result_dict is valid or not
    :return:
    """
    for key, value in result_check_dict.items():
        if result_dict.get(key) != value:
            raise APIAssertException(
                "value should be {right_value} but value was {wrong_value}".format(
                    right_value=value, wrong_value=result_dict.get(key)
                )
            )



import json.decoder
import sys
from json import dumps
from json import loads

from je_api_testka.utils.exception.exceptions_tag import api_test_cant_reformat_json_error
from je_api_testka.utils.exception.exceptions_tag import wrong_json_data_error
from je_api_testka.utils.exception.exceptions import APITesterJsonException


def __process_json(json_string: str, **kwargs):
    """
    :param json_string: full json str (not json type)
    :param kwargs: any another kwargs for dumps
    :return: reformat str
    """
    try:
        return dumps(loads(json_string), indent=4, sort_keys=True, **kwargs)
    except json.JSONDecodeError as error:
        print(wrong_json_data_error, file=sys.stderr)
        raise error
    except TypeError:
        try:
            return dumps(json_string, indent=4, sort_keys=True, **kwargs)
        except TypeError:
            raise APITesterJsonException(wrong_json_data_error)


def reformat_json(json_string: str, **kwargs):
    try:
        return __process_json(json_string, **kwargs)
    except APITesterJsonException:
        raise APITesterJsonException(api_test_cant_reformat_json_error)



import requests.exceptions
from requests.utils import dict_from_cookiejar


def get_api_response_data(response: requests.Response,
                          start_time: [str, float, int],
                          end_time: [str, float, int]) -> dict:
    """
    use requests response to create data dict
    :param response: requests response
    :param start_time: test start time
    :param end_time: test end time
    :return: data dict include [status_code, text, content, headers, history, encoding, cookies,
    elapsed, request_time_sec, request_method, request_url, request_body, start_time, end_time]
    """
    response_data = {
        "status_code": response.status_code,
        "text": response.text,
        "content": response.content,
        "headers": response.headers,
        "history": response.history,
        "encoding": response.encoding,
        "cookies": dict_from_cookiejar(response.cookies),
        "elapsed": response.elapsed,
        "request_time_sec": response.elapsed.total_seconds(),
        "request_method": response.request.method,
        "request_url": response.request.url,
        "request_body": response.request.body,
        "start_time": start_time,
        "end_time": end_time
    }
    try:
        if response_data.get("status_code") == 200:
            response_data.update({"json": response.json()})
        else:
            response_data.update({"json": None})
    except requests.exceptions.JSONDecodeError:
        response_data.update({"json": None})
    return response_data



import sys

import requests
from requests import Session
from requests import delete
from requests import get
from requests import head
from requests import options
from requests import patch
from requests import post
from requests import put

from je_api_testka.utils.exception.exceptions_tag import http_method_have_wrong_type
from je_api_testka.utils.exception.exceptions import APITesterException
from je_api_testka.utils.exception.exceptions_tag import wrong_http_method_error_message

_session = Session()

http_method_dict = {
    "get": get,
    "put": put,
    "patch": patch,
    "post": post,
    "head": head,
    "delete": delete,
    "options": options,
    "session_get": _session.get,
    "session_put": _session.put,
    "session_patch": _session.patch,
    "session_post": _session.post,
    "session_head": _session.head,
    "session_delete": _session.delete,
    "session_options": _session.options,
}


def get_http_method(http_method: str) -> [
            requests.get, requests.put, requests.patch, requests.post, requests.head, requests.delete,
            Session.get, Session.put, Session.patch, Session.post, Session.head, Session.head, Session.options
        ]:
    """
    :param http_method: what http method we use to api test
    :return: one of method in http_method_dict if not exists will raise exception
    """
    try:
        if type(http_method) is not str:
            raise APITesterException(wrong_http_method_error_message)
        http_method = str(http_method).lower()
        if http_method not in http_method_dict:
            raise APITesterException(http_method_have_wrong_type)
        return http_method_dict.get(http_method)
    except APITesterException as error:
        print(repr(error), file=sys.stderr)


def api_tester_method(http_method: str, test_url: str, **kwargs) -> requests.Response:
    """
    :param http_method: what http method we use to api test
    :param test_url: what url we want to test
    :param kwargs: use to setting
    :return: test response
    """
    response = get_http_method(http_method)
    if response is None:
        raise APITesterException(wrong_http_method_error_message)
    else:
        response = response(test_url, **kwargs)
    return response



# Unknown error
api_test_error_message = "API tester error"
wrong_http_method_error_message = "wrong http method"
http_method_have_wrong_type = "get wrong type param"
# get data error
api_test_get_data_error_message = "API tester can't get api data error"
# API Http Method error
api_test_get_error_message = "API tester get error"
api_test_get_json_error_message = "API tester get json error"
api_test_put_error_message = "API tester put error"
api_test_delete_error_message = "API tester delete error"
api_test_post_error_message = "API tester post error"
api_test_head_error_message = "API tester head error"
api_test_options_error_message = "API tester options error"
api_test_patch_error_message = "API tester patch error"
api_test_session_error_message = "API tester session error"
# API response code error
api_test_response_code_error = "API tester get wrong response code"
# API response body error
api_test_response_body_error = "API tester get wrong response body"
# API response timeout error
api_test_response_timeout_error = "API tester timeout"
# API response history error
api_test_response_history_error = "API tester history error"
# API response encoding error
api_test_response_encoding_error = "API tester encoding error"
# API response cookies error
api_test_response_cookies_error = "API tester cookies error"
# API response headers error
api_test_response_headers_error = "API tester headers error"
# API response content error
api_test_response_content_error = "API tester content error"
# API response text error
api_test_response_text_error = "API tester text error"
# API execute action error
api_test_execute_action_error = "can't execute action"
# API json error
api_test_cant_find_json_error = "can't find json"
api_test_cant_save_json_error = "can't save json"
api_test_cant_reformat_json_error = "can't reformat json is type right?"
api_test_cant_find_element_in_json_error = "can't find element in json"
api_test_json_type_error = "json type error"
wrong_json_data_error = "can't parser json"
# API XML error
api_test_cant_read_xml_error = "can't read xml"
api_test_xml_type_error = "xml type error"
# Executor error
executor_data_error = "executor receive wrong data"
executor_list_error = "executor receive wrong data list is none or wrong type"
# API data check
status_code_error = "request status code error"
status_text_error = "request text error"
status_content_error = "request content error"
status_headers_error = "request headers error"
status_history_error = "request history error"
status_encoding_error = "request encoding error"
status_cookies_error = "request cookies error"
status_elapsed_error = "request elapsed error"
status_requests_method_error = "requests method error"
status_requests_url_error = "requests url error"
status_requests_body_error = "requests body error"
# HTML
html_generate_no_data_tag = "record is None"




# test api method
from je_api_testka.requests_wrapper.request_method import test_api_method
# execute
from je_api_testka.utils.executor.action_executor import execute_action
from je_api_testka.utils.executor.action_executor import execute_files
# json
from je_api_testka.utils.json.json_file.json_file import write_action_json
from je_api_testka.utils.json.json_file.json_file import read_action_json
from je_api_testka.utils.json.json_format.json_process import reformat_json
# xml
from je_api_testka.utils.xml.xml_file.xml_file import XMLParser
from je_api_testka.utils.xml.xml_file.xml_file import reformat_xml_file
from je_api_testka.utils.xml.change_xml_structure.change_xml_structure import elements_tree_to_dict
from je_api_testka.utils.xml.change_xml_structure.change_xml_structure import dict_to_elements_tree
# test_record
from je_api_testka.utils.test_record.record_test_result_class import test_record
# html_output
from je_api_testka.utils.html_report.html_report_generate import generate_html
# file process
from je_api_testka.utils.file_process.get_dir_file_list import get_dir_files_as_list
from je_api_testka.utils.file_process.create_project_structure import create_template_dir
# graph
from je_api_testka.utils.create_graph.create_graph import make_tkinter_request_time_graph
